#!/usr/bin/env python3
# -*- coding: utf-8; -*-

import tbox
import tbox.gmsh as gmsh
import katoptron.utilities as u
import katoptron.eigenvalues as e
import katoptron.readers as r
import katoptron.convergence as c
from PyTrilinos import Teuchos
import shlex
import subprocess
import os
import numpy as np
import tarfile
import glob
from fwk.wutils import parseargs
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d
from katoptron.speedup import *


def CPU_time_all_cases(base_dir, n_cases, ensemble_sizes,
                       querylines, use_waves_timers, use_teuchos_timers, m,
                       use_prec=False):

    CPUs = np.zeros((n_cases, len(ensemble_sizes), len(querylines)))
    msh_size = np.zeros((n_cases,))

    for i in range(0, n_cases):
        msh_size[i] = np.loadtxt(
            base_dir + 'test_'+str(i)+'_s_1_no_prec_m_' + str(m) + '/msh_size.txt')
        for j in range(0, len(ensemble_sizes)):
            ensemble_size = ensemble_sizes[j]

            input_MC_file_dir = base_dir + 'test_' + \
                str(i)+'_s_'+str(ensemble_size)

            if use_prec:
                input_MC_file_dir = input_MC_file_dir + \
                    '_prec_m_' + str(m) + '/'
            else:
                input_MC_file_dir = input_MC_file_dir + \
                    '_no_prec_m_' + str(m) + '/'

            for k in use_waves_timers:
                CPUs[i, j, int(k)] = read_timers(
                    input_MC_file_dir + 'timers.txt', [querylines[int(k)]])[0]
            for k in use_teuchos_timers:
                CPUs[i, j, int(k)] = read_teuchos_timers(
                    input_MC_file_dir + 'teuchos_timers.txt',
                    [querylines[int(k)]])[0]

    return CPUs, msh_size


def plot_CPU_and_speedup(CPUs, msh_size, ensemble_sizes, index, querylines):
    plt.figure()
    for i in range(0, len(ensemble_sizes)):
        plt.loglog(3*msh_size, CPUs[:, i, index], '*-')

    plt.ylabel('CPU time [sec]')
    plt.xlabel('# dofs')
    plt.title(querylines[index])
    plt.grid(True, which="both")
    tmp_name = querylines[index].replace(": ", "_")
    tmp_name = tmp_name.replace(" ", "_")
    plt.savefig(tmp_name+'_CPU.png')

    np.savetxt(tmp_name+'_CPU.txt', CPUs[:, :, index])

    mc_all = np.zeros((len(ensemble_sizes), 2))
    for i in range(0, len(ensemble_sizes)):
        logX = np.log(3*msh_size)
        logY = np.log(CPUs[:, i, index])
        m, c = np.polyfit(logX, logY, 1)
        mc_all[i, 0] = m
        mc_all[i, 1] = c

    np.savetxt(tmp_name+'_CPU_mc.txt', mc_all)

    plt.figure()
    for i in range(1, len(ensemble_sizes)):
        plt.loglog(
            3*msh_size, (ensemble_sizes[i]*CPUs[:, 0, index] / CPUs[:, i, index]), '*-')

    x = 3*np.array([msh_size[0], msh_size[-1]])
    y = np.array([1., 1.])
    plt.grid(True, which="both")

    plt.ylabel('Speed-up')
    plt.xlabel('# dofs')
    plt.title(querylines[index])
    plt.savefig(tmp_name+'_speedup.png')
    for i in range(1, len(ensemble_sizes)):
        np.savetxt(tmp_name+'_speedup_'+str(
            ensemble_sizes[i])+'.txt', (ensemble_sizes[i]*CPUs[:, 0, index] / CPUs[:, i, index]))


def main():
    base_dir = '/Users/kimliegeois/Desktop/results/'
    n_cases = 50

    ensemble_sizes = [1, 8, 16, 24, 32]

    querylines = [
        'total', 'write results', 'computeMatrices',
        'Belos: Operation Op*x', 'Belos: Operation Prec*x',
        'Belos: DGKS[2]: Orthogonalization',
        'Belos: DGKS[2]: Ortho (Inner Product)',
        'Belos: DGKS[2]: Ortho (Norm)', 'Belos: DGKS[2]: Ortho (Update)',
        'Belos: PseudoBlockGmresSolMgr total solve time'
    ]
    use_waves_timers = np.array([0, 1, 2], dtype=int)
    use_teuchos_timers = np.array(
        [3, 4, 5, 6, 7, 8, 9],
        dtype=int)

    m = 100

    CPUs, msh_size = CPU_time_all_cases(base_dir, n_cases, ensemble_sizes,
                                        querylines, use_waves_timers, use_teuchos_timers, m,
                                        use_prec=False)

    CPUs[:, :, 0] = CPUs[:, :, 0] - CPUs[:, :, 1]

    np.savetxt('N_dofs.txt', 3*msh_size)
    #print CPUs
    #print msh_size

    for i in range(0, 10):
        plot_CPU_and_speedup(CPUs, msh_size, ensemble_sizes, i, querylines)


if __name__ == "__main__":
    main()
