#!/usr/bin/env python3
# -*- coding: utf-8; -*-






import katoptron as m
import tbox
import tbox.gmsh as gmsh
import katoptron.utilities as u
from PyTrilinos import Teuchos
import shlex
import subprocess
import os
import numpy as np
import tarfile
import glob
from fwk.wutils import parseargs
from katoptron.Juelich.UQ.halton import *


def tar_results(directory, filename, rank):
    os.chdir(directory)
    if rank == 0:
        tf = tarfile.open(filename + ".tar.gz", mode="w:gz")
        for file in glob.glob('*'):
            if file.endswith(".tar.gz"):
                print(file + ' is not included')
            else:
                tf.add(file)
        tf.close()


def model(msh, E, p, ensemble_size, work_dir):
    comm, rank, size = m.utilities.import_MPI()

    partitioned_mesh_name = 'beam_k.msh'
    file_dir = os.path.dirname(__file__)
    current_dir = os.getcwd()

    pbl = m.Problem(msh, comm)

    nu = 0.29 * np.ones(ensemble_size)
    k = 0
    d = 0

    m.Medium(pbl, "Beam", "Mat 1", E, nu, k, d)
    m.Medium(pbl, "Rigid body", "Mat 1", E, nu, k, d)

    dx = np.zeros(ensemble_size)
    dy = np.zeros(ensemble_size)
    dz = np.zeros(ensemble_size)
    dT = np.zeros(ensemble_size)
    zero = np.zeros(ensemble_size)
    m.Dirichlet(pbl, "Rigid clamped surface",
                "Clamped 1", 1, dx, 1, dy, 1, dz, 0, dT)
    m.Dirichlet(pbl, "Clamped end", "Clamped 1", 1, dx, 1, dy, 1, dz, 0, dT)
    m.Dirichlet(pbl, "Free end", "Clamped 1", 1, 0.,
                0, 0., 0, 0., 0, 0., ensemble_size)

    m.Neumann(pbl, "Loaded surface", "Load 1", 0, zero,
              0, zero, 1, -p, 0, zero, ensemble_size)

    norm1 = tbox.Vector3d(0, 0, -1)
    norm2 = tbox.Vector3d(0, 0, 1)
    dg = np.zeros(ensemble_size)

    a = 49.95
    initiallyClosedNodesList = np.array([], dtype=np.float)
    for el in msh.ntags["Beam contact zone"].elems:
        for n1 in el.nodes:
            node_id = n1.no-1
            if msh.nodes[node_id].pos[0] >= a:
                initiallyClosedNodesList = np.append(
                    initiallyClosedNodesList, node_id)

    cont1 = m.Contact(pbl, "Beam contact zone", "contact", norm1, dg)
    cont1.setInitialyOpen()
    cont1.setInitiallyClosedNodes(initiallyClosedNodesList)
    cont1.setDualShapeFunctions()
    cont1.setSlaveDirichlet(
        work_dir+"/"+partitioned_mesh_name, "Beam contact Dirichlet")
    cont1.setSignoriniContact()
    cont1.setMaster(pbl, "Rigid surface", norm2)

    solverList = Teuchos.ParameterList()
    solverList['Convergence Tolerance'] = 10**(-8)
    solverList['Maximum Iterations'] = 500
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    muelu_xml_in = file_dir + '/../../preconditioners/SIMPLE_gs_direct_lvl_MT.xml.in'
    old_strings = ['$BLOCK_00_STRIDING_INFO', '$BLOCK_11_STRIDING_INFO',
                   '$BLOCK_00_GS_sweeps', '$BLOCK_00_GS_damping',
                   '$SCHUR_omega', '$SimpleSmoother_damping', "$N_LVLS", "$N_SIZES", "$EXPORT_DATA", "$SimpleSmoother_sweeps"]
    new_strings = ['3', '1', '3', '0.8', '0.8', '1', '2', '18', "true", '1']

    u.replace_strings(muelu_xml_in, current_dir +
                      '/SIMPLE_MG.xml', old_strings, new_strings)

    solverList["MueLu xml file name"] = current_dir+'/SIMPLE_MG.xml'
    solverList["convert MueLu xml file"] = True

    solverList['Use preconditioner'] = True

    solverList['type'] = "BlockGmres"
    solverList['Maximum active set iteration'] = 10
    solverList['Write txt files'] = True
    solverList['Print Teuchos timers'] = True

    slv = m.IterativeSolver(pbl, solverList, 3, ensemble_size)

    if rank == 0:
        np.savetxt('E.txt', E)
        np.savetxt('p.txt', p)

    slv.start()

    f = open('timers.txt', 'w')
    print(slv.getTimers(), file=f)
    f.closed


def evaluate_all_ensembles(ensemble_sizes, N, msh, comm, rank, size,
                           file_dir, work_dir, E_vec, p_vec):
    """
    This function loop over the ensemble sizes,
    and the ensembles
    """
    for ensemble_size in ensemble_sizes:
        directory = str(ensemble_size)
        u.mkdir_MPI(directory, comm, rank, size)
        os.chdir(directory)

        N_ensemble = N // ensemble_size

        ensemble_index_min = 0
        for ensemble_index in range(ensemble_index_min, N_ensemble):
            directory = str(ensemble_index)
            u.mkdir_MPI(directory, comm, rank, size)
            os.chdir(directory)

            ensemble_index_start = ensemble_index * ensemble_size
            ensemble_index_end = ensemble_index_start + ensemble_size

            E = E_vec[ensemble_index_start:ensemble_index_end]

            p = p_vec[ensemble_index_start:ensemble_index_end]

            model(msh, E, p, ensemble_size, work_dir)

            os.chdir('..')
        os.chdir('..')


def main():
    comm, rank, size = m.utilities.import_MPI()

    args = parseargs()

    geo_name = 'beam.geo'

    larger_mesh = True

    par = {}
    par['L'] = 50.
    par['W'] = 5.
    par['H'] = 5.
    par['d'] = 1.
    if larger_mesh:
        par['n_L'] = 200
        par['n_hW'] = 5
        par['n_hH'] = 5
    else:
        par['n_L'] = 60
        par['n_hW'] = 3
        par['n_hH'] = 3

    import shlex
    import subprocess
    import os

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    pars = {'L': par['L'], 'W': par['W'], 'H': par['H'], 'd': par['d'],
            'n_L': par['n_L'], 'n_hW': par['n_hW'], 'n_hH': par['n_hH']}

    msh, mesh_name, partitioned_mesh_name = u.read_mesh_MPI(
        geo_name, file_dir, work_dir, comm, rank, size, pars)

    for n1 in msh.nodes:
        n1.row = n1.no-1

    N = 640
    p_min = 0.5
    p_max = 2.5
    E_min = 20500
    E_max = 21500

    n_rand = 2
    halton_points = np.zeros((n_rand, N))
    E_vec = np.zeros((N, ))
    p_vec = np.zeros((N, ))
    for i in range(0, N):
        halton_points[:, i] = halton(i, n_rand)

    E_vec = E_min + halton_points[0, :] * (E_max-E_min)
    p_vec = p_min + halton_points[1, :] * (p_max-p_min)

    grouped = False

    if grouped:
        order = np.loadtxt(file_dir+'/ordering.txt', dtype=int)
        E_vec = E_vec[order[0:N]]
        p_vec = p_vec[order[0:N]]

    if rank == 0:
        np.savetxt('E_vec.txt', E_vec)
        np.savetxt('p_vec.txt', p_vec)

    ensemble_sizes = [1, 8, 16, 32]

    nThreads = u.Initialize_Kokkos()
    evaluate_all_ensembles(ensemble_sizes, N, msh, comm, rank, size,
                           file_dir, work_dir, E_vec, p_vec)

    u.Finalize_Kokkos()

    tar_results(work_dir, 'all_results', rank)


if __name__ == "__main__":
    main()
