#!/usr/bin/env python3
# -*- coding: utf-8; -*-

from scipy.sparse.linalg import spsolve
from scipy.sparse.linalg import eigs
from scipy.linalg import eig
from scipy.sparse import spdiags
import numpy as np
import matplotlib.pyplot as plt
import katoptron.convergence as c


def build_K(n, L, E, A, sym=True):
    h = (L / (n - 1))
    diag_data = A * E * ((1 / h**2)) * 2. * np.ones((n, ))
    diag_data[0] = 1.
    diag_data[-1] = A * E * ((1 / h**2)) * 1.
    off_data_1 = A * E * ((1 / h**2)) * (-1) * np.ones((n, ))
    if sym:
        off_data_1[0:1] = 0.
    off_data_2 = A * E * ((1 / h**2)) * (-1) * np.ones((n, ))
    off_data_2[0:2] = 0.

    data = np.array([diag_data, off_data_1, off_data_2])
    diags = np.array([0, -1, 1])
    K = spdiags(data, diags, n, n).tocsr()
    return K


def analytical_eigenfunctions(n, L, E, A, n_eigenfunctions=5):
    if n < n_eigenfunctions + 1:
        n_eigenfunctions = n - 1
    values = np.zeros((n, ))
    vecs = np.zeros((n, n_eigenfunctions))

    h = (L / (n - 1))
    x = np.linspace(0, L, n)

    c = np.arange(0, n_eigenfunctions, 1) + 0.5
    values = A * E * (c * 3.14 / L)**2

    for i in range(0, n_eigenfunctions):
        vecs[:, i] = np.sin((c[i] * np.pi * x / L))
        norm = np.linalg.norm(vecs[:, i])
        vecs[:, i] /= norm
    return values, vecs


def numerical_eigenfunctions(n, L, E, A, n_eigenfunctions=5):
    if n < n_eigenfunctions + 1:
        n_eigenfunctions = n - 1
    K = build_K(n, L, E, A)

    vals, vecs = eig(K.toarray())
    return np.real(vals[0:n_eigenfunctions]), vecs[:, 0:n_eigenfunctions]


def omegas_lambdas_nf(n, L, nE, A, nf):
    N = len(nE)
    norm = 0.
    for i in range(0, N):
        norm += np.linalg.norm(nf[i])**2
    norm = np.sqrt(norm)
    lambdas_all = np.array([])
    omegas_all = np.array([])
    for i in range(0, N):
        nf_tmp = (nf[i] / norm)
        lambdas, vecs = analytical_eigenfunctions(n, L, nE[i], A, n - 1)
        print(vecs)
        c = np.dot(vecs.T, nf_tmp)

        omegas = np.zeros((len(c), ))
        for i in range(0, len(omegas)):
            omegas[i] = np.linalg.norm(c[i])**2

        lambdas_all = np.append(lambdas_all, lambdas)
        omegas_all = np.append(omegas_all, omegas)

    indices = np.argsort(lambdas_all)
    lambdas = lambdas_all[indices]
    omegas = omegas_all[indices]

    return lambdas, omegas


def omegas_lambdas_f(n, L, E, A, f):
    lambdas, vecs = analytical_eigenfunctions(n, L, E, A, n - 1)
    norm = np.linalg.norm(f)
    f_tmp = (f / norm)
    c = np.dot(vecs.T, f_tmp)

    omegas = np.zeros((len(c), ))
    for i in range(0, len(omegas)):
        omegas[i] = np.linalg.norm(c[i])**2

    return lambdas, omegas


def build_f(n, L, lF, F):
    h = (L / (n - 1))
    iF = int(np.round((lF / h)) + 1)
    f = np.zeros((n, ))
    f[iF] = (F / h)
    return f


def solve_Kf(n, L, lF, F, E, A):
    sym = True

    K = build_K(n, L, E, A, sym)
    f = build_f(n, L, lF, F)

    u = spsolve(K, f)
    return u


def write_on_disk(name, res):
    i_max = len(res)
    for i in range(0, i_max):
        if res[i] == 0.:
            i_max = i + 1
            break
    tmp = np.zeros((i_max, 2))
    tmp[:, 1] = res[0:i_max]
    tmp[:, 0] = np.arange(0, i_max)
    np.savetxt(name, tmp)


def main():
    n = 7
    L = 1.
    E_1 = 1.
    E_2 = 1.5
    A = 1.
    F = 1.

    x = np.linspace(0, L, n)

    f_1 = build_f(n, L, 0.5, F)
    f_2 = build_f(n, L, 0.8, F)

    print(f_1)
    print(f_2)

    lambdas_11, omegas_11 = omegas_lambdas_f(n, L, E_1, A, f_1)
    lambdas_12, omegas_12 = omegas_lambdas_f(n, L, E_1, A, f_2)

    lambdas_21, omegas_21 = omegas_lambdas_f(n, L, E_2, A, f_1)
    lambdas_22, omegas_22 = omegas_lambdas_f(n, L, E_2, A, f_2)

    residual_11 = c.compute_convergence_serial_normal(20, n - 1, omegas_11,
                                                      lambdas_11)
    residual_12 = c.compute_convergence_serial_normal(20, n - 1, omegas_12,
                                                      lambdas_12)
    residual_21 = c.compute_convergence_serial_normal(20, n - 1, omegas_21,
                                                      lambdas_21)
    residual_22 = c.compute_convergence_serial_normal(20, n - 1, omegas_22,
                                                      lambdas_22)

    lambdas_1, omegas_1 = omegas_lambdas_nf(n, L, [E_1, E_1], A, [f_1, f_2])
    lambdas_2, omegas_2 = omegas_lambdas_nf(n, L, [E_2, E_2], A, [f_1, f_2])
    lambdas_3, omegas_3 = omegas_lambdas_nf(n, L, [E_1, E_2], A, [f_1, f_1])
    lambdas_4, omegas_4 = omegas_lambdas_nf(n, L, [E_1, E_2], A, [f_2, f_2])
    lambdas_5, omegas_5 = omegas_lambdas_nf(n, L, [E_1, E_2], A, [f_1, f_2])

    residual_1 = c.compute_convergence_serial_normal(20, 2 * (n - 1), omegas_1,
                                                     lambdas_1)
    residual_2 = c.compute_convergence_serial_normal(20, 2 * (n - 1), omegas_2,
                                                     lambdas_2)
    residual_3 = c.compute_convergence_serial_normal(20, 2 * (n - 1), omegas_3,
                                                     lambdas_3)
    residual_4 = c.compute_convergence_serial_normal(20, 2 * (n - 1), omegas_4,
                                                     lambdas_4)
    residual_5 = c.compute_convergence_serial_normal(20, 2 * (n - 1), omegas_5,
                                                     lambdas_5)

    write_on_disk('res_11.txt', residual_11)
    write_on_disk('res_12.txt', residual_12)
    write_on_disk('res_21.txt', residual_21)
    write_on_disk('res_22.txt', residual_22)
    write_on_disk('res_1.txt', residual_1)
    write_on_disk('res_2.txt', residual_2)
    write_on_disk('res_3.txt', residual_3)
    write_on_disk('res_4.txt', residual_4)
    write_on_disk('res_5.txt', residual_5)

    plt.figure()
    plt.semilogy(residual_11)
    plt.semilogy(residual_22)
    plt.semilogy(residual_1)
    '''
    plt.figure()
    plt.subplot(221)
    plt.plot(lambdas_11, '-*')
    plt.subplot(222)
    plt.plot(lambdas_12, '-*')
    plt.subplot(223)
    plt.plot(lambdas_21, '-*')
    plt.subplot(224)
    plt.plot(lambdas_22, '-*')
    plt.figure()
    plt.subplot(221)
    plt.plot(omegas_11, '-*')
    plt.subplot(222)
    plt.plot(omegas_12, '-*')
    plt.subplot(223)
    plt.plot(omegas_21, '-*')
    plt.subplot(224)
    plt.plot(omegas_22, '-*')

    plt.figure()
    plt.plot(lambdas_44, '-*')
    plt.figure()
    plt.plot(omegas_44, '-*')
    '''
    plt.show()


if __name__ == "__main__":
    main()
